@@ -50,7 +50,6 @@ def write_default_ansible_cfg(work_dir,
                os.path.join(work_dir, 'ansible.log'))
     config.set('defaults', 'forks', '25')
     config.set('defaults', 'timeout', '30')
-    config.set('defaults', 'gather_timeout', '30')
 
     # mistral user has no home dir set, so no place to save a known hosts file
     config.set('ssh_connection', 'ssh_args',
@@ -95,7 +94,7 @@ class AnsibleAction(actions.Action):
         self.hosts = self._kwargs_for_run.pop('hosts', None)
         self.module = self._kwargs_for_run.pop('module', None)
         self.module_args = self._kwargs_for_run.pop('module_args', None)
-        if self.module_args and isinstance(self.module_args, dict):
+        if self.module_args:
             self.module_args = json.dumps(self.module_args)
         self.limit_hosts = self._kwargs_for_run.pop('limit_hosts', None)
         self.remote_user = self._kwargs_for_run.pop('remote_user', None)
@@ -319,8 +318,6 @@ class AnsiblePlaybookAction(base.TripleOAction):
             'blacklisted_hostnames', [])
         self.override_ansible_cfg = self._kwargs_for_run.pop(
             'override_ansible_cfg', None)
-        self.command_timeout = self._kwargs_for_run.pop(
-            'command_timeout', None)
 
     @property
     def work_dir(self):
@@ -528,9 +525,6 @@ class AnsiblePlaybookAction(base.TripleOAction):
                     'OS_AUTH_TOKEN': security_ctx.auth_token,
                     'OS_PROJECT_NAME': security_ctx.project_name})
 
-            if self.command_timeout:
-                command = ['timeout', self.command_timeout] + command
-
             command = [str(c) for c in command]
 
             if self.reproduce_command:
@@ -563,8 +557,7 @@ class AnsiblePlaybookAction(base.TripleOAction):
                 lines = []
                 for line in iter(process.stdout.readline, b''):
                     lines.append(line)
-                    if not self.trash_output:
-                        stdout.append(line)
+                    stdout.append(line)
                     if time.time() - start > 30:
                         self.post_message(queue, ''.join(lines))
                         lines = []
@@ -576,6 +569,9 @@ class AnsiblePlaybookAction(base.TripleOAction):
                 # stdout we don't know the difference. To keep the return dict
                 # similar there is an empty stderr. We can use the return code
                 # to determine if there was an error.
+                if self.trash_output:
+                    stdout = []
+                    stderr = ""
                 return {"stdout": "".join(stdout), "returncode": returncode,
                         "stderr": ""}
 
