@@ -127,13 +127,11 @@ class UpdateParametersAction(templates.ProcessTemplatesAction):
 
     def __init__(self, parameters,
                  container=constants.DEFAULT_CONTAINER_NAME,
-                 key=constants.DEFAULT_PLAN_ENV_KEY,
-                 validate=True):
+                 key=constants.DEFAULT_PLAN_ENV_KEY):
         super(UpdateParametersAction, self).__init__()
         self.container = container
         self.parameters = parameters
         self.key = key
-        self.validate = validate
 
     def run(self, context):
         swift = self.get_object_client(context)
@@ -166,9 +164,8 @@ class UpdateParametersAction(templates.ProcessTemplatesAction):
         if isinstance(processed_data, actions.Result):
             return processed_data
 
+        processed_data['show_nested'] = True
         env = plan_utils.get_env(swift, self.container)
-        if not self.validate:
-            return env
 
         params = env.get('parameter_defaults')
         fields = {
@@ -397,7 +394,7 @@ class GenerateFencingParametersAction(base.TripleOAction):
             if "ports" in node:
                 # Not all Ironic drivers present a MAC address, so we only
                 # capture it if it's present
-                mac_addr = node['ports'][0]['address'].lower()
+                mac_addr = node['ports'][0]['address']
                 node_data["host_mac"] = mac_addr
 
                 # If the MAC isn't in the hostmap, this node hasn't been
@@ -414,14 +411,9 @@ class GenerateFencingParametersAction(base.TripleOAction):
                 # New-style hardware types (ipmi, etc)
                 driver_proto = node['pm_type']
 
-            if driver_proto in {'ipmi', 'ipmitool', 'drac', 'idrac', 'ilo',
-                                'redfish'}:
+            if driver_proto in {'ipmi', 'ipmitool', 'drac', 'idrac', 'ilo'}:
                 # IPMI fencing driver
-                if driver_proto == "redfish":
-                    node_data["agent"] = "fence_redfish"
-                    params["systems_uri"] = node["pm_system_id"]
-                else:
-                    node_data["agent"] = "fence_ipmilan"
+                node_data["agent"] = "fence_ipmilan"
                 params["ipaddr"] = node["pm_addr"]
                 params["passwd"] = node["pm_password"]
                 params["login"] = node["pm_user"]
@@ -430,11 +422,6 @@ class GenerateFencingParametersAction(base.TripleOAction):
                         hostmap[mac_addr]["compute_name"]
                 if "pm_port" in node:
                     params["ipport"] = node["pm_port"]
-                if "redfish_verify_ca" in node:
-                    if node["redfish_verify_ca"] == "false":
-                        params["ssl_insecure"] = "true"
-                    else:
-                        params["ssl_insecure"] = "false"
                 if self.ipmi_lanplus:
                     params["lanplus"] = self.ipmi_lanplus
                 if self.delay:
@@ -443,20 +430,6 @@ class GenerateFencingParametersAction(base.TripleOAction):
                     params["cipher"] = self.ipmi_cipher
                 if self.ipmi_level:
                     params["privlvl"] = self.ipmi_level
-            elif driver_proto in {'staging-ovirt'}:
-                # fence_rhevm
-                node_data["agent"] = "fence_rhevm"
-                params["ipaddr"] = node["pm_addr"]
-                params["passwd"] = node["pm_password"]
-                params["login"] = node["pm_user"]
-                params["port"] = node["pm_vm_name"]
-                params["ssl"] = 1
-                params["ssl_insecure"] = 1
-                if hostmap:
-                    params["pcmk_host_list"] = \
-                        hostmap[mac_addr]["compute_name"]
-                if self.delay:
-                    params["delay"] = self.delay
             else:
                 error = ("Unable to generate fencing parameters for %s" %
                          node["pm_type"])
