@@ -110,7 +110,8 @@ class DeletePlanAction(base.TripleOAction):
         # heat throws HTTPNotFound if the stack is not found
         try:
             stack = self.get_orchestration_client(context).stacks.get(
-                self.container, resolve_outputs=False)
+                self.container
+            )
         except heatexceptions.HTTPNotFound:
             pass
         else:
