@@ -44,3 +44,7 @@ class OvercloudRcTest(base.TestCase):
         self.assertIn(overcloudrc.CLOUDPROMPT, result['overcloudrc.v3'])
         self.assertIn("OS_AUTH_TYPE=password", result['overcloudrc'])
         self.assertIn("OS_AUTH_TYPE=password", result['overcloudrc.v3'])
+        self.assertIn("OS_AUTH_URL=http://foo.com:8000/",
+                      result['overcloudrc'])
+        self.assertIn("OS_AUTH_URL=http://foo.com:8000/",
+                      result['overcloudrc.v3'])
