@@ -150,7 +150,9 @@ class Config(object):
     def snapshot_config_dir(self, repo, commit_message):
         if repo.is_dirty(untracked_files=True):
             self.log.info('Snapshotting {}'.format(repo.working_dir))
-            repo.index.add('*')
+            # Use repo.git.add directly as repo.index.add defaults to forcing
+            # commit of ignored files, which we don't want.
+            repo.git.add('.')
             commit = repo.index.commit(commit_message)
             self.log.info('Created commit {}'.format(commit.hexsha))
         else:
@@ -196,6 +198,7 @@ class Config(object):
         # Get role data:
         role_data = self.stack_outputs.get('RoleData', {})
         role_group_vars = self.stack_outputs.get('RoleGroupVars', {})
+        allnodes_vars = self.stack_outputs.get('AllNodesConfig', {})
         for role_name, role in six.iteritems(role_data):
             role_path = os.path.join(config_dir, role_name)
             self._mkdir(role_path)
@@ -207,6 +210,14 @@ class Config(object):
                     filepath = os.path.join(role_path, 'step_config.pp')
                     with self._open_file(filepath) as step_config:
                         step_config.write(role[config])
+                elif config == 'ansible_group_vars':
+                    # NOTE(aschultz): ansible group vars are for specific role
+                    # services so we merge them in with the others so they
+                    # end up in the role vars. This means the last var
+                    # definition wins and will override them all.
+                    if role_name not in role_group_vars:
+                        role_group_vars[role_name] = {}
+                    role_group_vars[role_name].update(role[config])
                 else:
                     if 'upgrade_tasks' in config:
                         filepath = os.path.join(role_path, '%s_playbook.yaml' %
@@ -253,6 +264,21 @@ class Config(object):
         server_roles = {}
 
         for deployment in deployments_data:
+            # Check if the deployment value is the resource name. If that's the
+            # case, Heat did not create a physical_resource_id for this
+            # deployment since it does not trigger on this stack action. Such
+            # as a deployment that only triggers on DELETE, but this is a stack
+            # create. If that's the case, just skip this deployment, otherwise
+            # it will result in a Not found error if we try and query the
+            # deployment API for this deployment.
+            if deployment.attributes['value'].get('deployment') == \
+                    'TripleOSoftwareDeployment':
+                warnings.warn('Skipping deployment %s because it has no '
+                              'valid uuid (physical_resource_id) '
+                              'associated.' %
+                              deployment.physical_resource_id)
+                continue
+
             server_id = deployment.attributes['value']['server']
             config_dict = self.get_config_dict(deployment)
 
@@ -353,6 +379,13 @@ class Config(object):
 
         group_vars_dir = os.path.join(config_dir, 'group_vars')
         self._mkdir(group_vars_dir)
+        # Write allnodes_vars to 'overcloud' group as it doesn't apply
+        # to the undercloud, and 'overcloud' is the inventory group
+        # for all overcloud nodes, regardless of the --stack name.
+        filepath = os.path.join(group_vars_dir, 'overcloud.yaml')
+        with self._open_file(filepath) as vars_file:
+            yaml.safe_dump(allnodes_vars, vars_file, default_flow_style=False)
+
         host_vars_dir = os.path.join(config_dir, 'host_vars')
         self._mkdir(host_vars_dir)
 
@@ -397,12 +430,12 @@ class Config(object):
                                "config-download." % d['deployment_name'])
                     warnings.warn(message, DeprecationWarning)
 
-                with open(deployment_path, 'w') as f:
+                with open(deployment_path, 'wb') as f:
                     template_data = deployment_template.render(
                         deployment=d,
                         server_id=server_ids[server])
                     self.validate_config(template_data, deployment_path)
-                    f.write(template_data)
+                    f.write(template_data.encode('utf-8'))
 
         for role, deployments in role_deployment_names.items():
             group_var_role_path = os.path.join(group_vars_dir, role)
