@@ -14,16 +14,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import dateutil.parser
 import logging
 import os
 import tempfile
 
-from tripleo_common import constants
-from tripleo_common.utils import tarball
-
+import six
 from swiftclient.service import SwiftError
 from swiftclient.service import SwiftUploadObject
 
+from tripleo_common.utils import tarball
+
 LOG = logging.getLogger(__name__)
 
 
@@ -33,18 +34,11 @@ def empty_container(swiftclient, name):
 
     if name in container_names:
         headers, objects = swiftclient.get_container(name)
-        # ensure container is a plan
-        if headers.get(constants.TRIPLEO_META_USAGE_KEY) != 'plan':
-            error_text = ("The {name} container does not contain a "
-                          "TripleO deployment plan and was not "
-                          "deleted.".format(name=name))
-            raise ValueError(error_text)
-        else:
-            # FIXME(rbrady): remove delete_object loop when
-            # LP#1615830 is fixed.  See LP#1615825 for more info.
-            # delete files from plan
-            for o in objects:
-                swiftclient.delete_object(name, o['name'])
+        # FIXME(rbrady): remove delete_object loop when
+        # LP#1615830 is fixed.  See LP#1615825 for more info.
+        # delete files from plan
+        for o in objects:
+            swiftclient.delete_object(name, o['name'])
     else:
         error_text = "The {name} container does not exist.".format(name=name)
         raise ValueError(error_text)
@@ -57,25 +51,42 @@ def delete_container(swiftclient, name):
     except ValueError as e:
         # ValueError is raised when we can't find the container, which means
         # that it's already deleted.
-        LOG.info(e.message)
+        LOG.info(six.text_type(e))
 
 
-def download_container(swiftclient, container, dest):
+def download_container(swiftclient, container, dest,
+                       overwrite_only_newer=False):
     """Download the contents of a Swift container to a directory"""
 
     objects = swiftclient.get_container(container)[1]
 
     for obj in objects:
+        is_newer = False
         filename = obj['name']
         contents = swiftclient.get_object(container, filename)[1]
         path = os.path.join(dest, filename)
         dirname = os.path.dirname(path)
+        already_exists = os.path.exists(path)
 
-        if not os.path.exists(dirname):
-            os.makedirs(dirname)
+        if already_exists:
+            last_mod_swift = int(dateutil.parser.parse(
+                obj['last_modified']).strftime('%s'))
+            last_mod_disk = int(os.path.getmtime(path))
 
-        with open(path, 'w') as f:
-            f.write(contents)
+            if last_mod_swift > last_mod_disk:
+                is_newer = True
+
+        # write file if `overwrite_only_newer` is not set,
+        # or if file does not exist at destination,
+        # or if we found a newer file at source
+        if (not overwrite_only_newer
+                or not already_exists
+                or (overwrite_only_newer and is_newer)):
+            if not os.path.exists(dirname):
+                os.makedirs(dirname)
+
+            with open(path, 'w') as f:
+                f.write(contents)
 
 
 def create_container(swiftclient, container):
